import { DataModel } from '@phosphor/datagrid';
import { Message } from '@phosphor/messaging';
import { Widget } from '@phosphor/widgets';
import { ISignal } from '@phosphor/signaling';
import { MainAreaWidget } from '@jupyterlab/apputils';
import { CodeEditor, CodeEditorWrapper } from '@jupyterlab/codeeditor';
import { IMapDConnectionData } from './connection';
export declare class MapDExplorer extends MainAreaWidget<MapDGrid> {
    /**
     * Construct a new MapDExplorer widget.
     */
    constructor(options: MapDExplorer.IOptions);
    /**
     * Get a reference to the input editor.
     */
    readonly input: CodeEditorWrapper;
    /**
     * Handle the DOM events for the widget.
     *
     * @param event - The DOM event sent to the widget.
     *
     * #### Notes
     * This method implements the DOM `EventListener` interface and is
     * called in response to events on the main area widget's node. It should
     * not be called directly by user code.
     */
    handleEvent(event: KeyboardEvent): void;
    /**
     * Handle `after-attach` messages for the widget.
     */
    protected onAfterAttach(msg: Message): void;
    /**
     * Handle `before-detach` messages for the widget.
     */
    protected onBeforeDetach(msg: Message): void;
    /**
     * Handle `'activate-request'` messages.
     */
    protected onActivateRequest(msg: Message): void;
    /**
     * Focus the toolbar editor widget.
     */
    private _focusInput;
}
/**
 * A namespace for MapDExplorer statics.
 */
export declare namespace MapDExplorer {
    /**
     * Options for creating a new MapDExplorer.
     */
    interface IOptions {
        /**
         * An editor factory for the SQL editor widget.
         */
        editorFactory: CodeEditor.Factory;
        /**
         * An optional initial connection data structure.
         */
        connection?: IMapDConnectionData;
    }
}
/**
 * A widget that hosts a phosphor grid with a MapD dataset.
 */
export declare class MapDGrid extends Widget {
    /**
     * Construct a new MapDGrid widget.
     */
    constructor(connection?: IMapDConnectionData);
    /**
     * The current connection data for the viewer.
     */
    connection: IMapDConnectionData;
    /**
     * The query for the viewer.
     */
    query: string;
    /**
     * A change signal emitted when the connection or
     * query data change.
     */
    readonly onModelChanged: ISignal<MapDGrid, void>;
    /**
     * Update the underlying data model with a new query and connection.
     *
     * If the update fails, either due to a connection failure or a query
     * validation failure, it shows the error in the panel.
     */
    private _updateModel;
    private _model;
    private _grid;
    private _gridStyle;
    private _content;
    private _error;
    private _onModelChanged;
}
/**
 * A data model for a query.
 */
export declare class MapDTableModel extends DataModel {
    /**
     * Construct a new data model.
     */
    constructor();
    /**
     * Get the number of rows for the model.
     */
    rowCount(region: DataModel.RowRegion): number;
    /**
     * Get the number of columns for the model.
     */
    columnCount(region: DataModel.ColumnRegion): number;
    /**
     * The current connection data for the model.
     */
    readonly connection: IMapDConnectionData | undefined;
    /**
     * The current query for the viewer.
     */
    readonly query: string;
    /**
     * Get data from the model.
     */
    data(region: DataModel.CellRegion, row: number, column: number): any;
    /**
     * Update the model with new connection data or a new query.
     *
     * @param connection - the connection data to use.
     *
     * @param query - the query to use.
     *
     * @returns a promsise that resolves when the model has been updated,
     *   and the connection and query data have been validated. It throws
     *   an error if the validation fails.
     */
    updateModel(connection: IMapDConnectionData, query: string): Promise<void>;
    /**
     * Reset the model. Should be called when either
     * the query or the connection data change.
     */
    private _updateModel;
    /**
     * Fetch a block with a given index into memory.
     */
    private _fetchBlock;
    /**
     * Free references to a block when it is no longer needed.
     */
    private _freeBlock;
    /**
     * If we are not chunking the data, then just load the whole thing,
     * limited by DEFAULT_LIMIT.
     */
    private _fetchDataset;
    private _query;
    private _connection;
    private _fieldNames;
    private _dataBlocks;
    private _dataset;
    private _pending;
    private _tableLength;
    private _maxBlock;
    private _streaming;
}
