"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const widgets_1 = require("@phosphor/widgets");
const widget_1 = require("./widget");
const vega_lite_1 = require("./vega-lite");
require("../style/index.css");
/**
 * The MIME type for backend-rendered MapD.
 */
exports.MIME_TYPE = 'application/vnd.mapd.vega+json';
/**
 * The MIME type for png data.
 */
exports.IMAGE_MIME = 'image/png';
/**
 * A class for rendering a MapD-generated image.
 */
class RenderedMapD extends widgets_1.Widget {
    constructor() {
        super(...arguments);
        this._widget = null;
    }
    /**
     * Render MapD image into this widget's node.
     */
    renderModel(model) {
        // If we have already rendered a widget, dispose of it.
        if (this._widget) {
            this._widget.parent = null;
            this._widget.dispose();
            this._widget = null;
        }
        // If there is png image data in the mimebundle,
        // we can render that instead of making another
        // request to the backend.
        const imageData = model.data[exports.IMAGE_MIME];
        if (imageData) {
            this.node.appendChild(Private.createImageNode(imageData));
            return Promise.resolve(void 0);
        }
        // Get the data from the mimebundle
        const data = model.data[exports.MIME_TYPE];
        const { connection, vega, vegalite } = data;
        // Create a new MapDVega
        this._widget = new widget_1.MapDVega(vega || vega_lite_1.compileToVega(vegalite), connection, vegalite);
        this.node.appendChild(this._widget.node);
        return this._widget.renderedImage.then(data => {
            // Set the mime data for the png.
            // This allows us to re-use the image if
            // we are loading from disk.
            model.setData({
                data: Object.assign({ 'image/png': data }, model.data),
                metadata: model.metadata
            });
            return void 0;
        });
    }
}
exports.RenderedMapD = RenderedMapD;
/**
 * A mime renderer factory for mapd-vega data.
 */
exports.rendererFactory = {
    safe: false,
    mimeTypes: [exports.MIME_TYPE],
    defaultRank: 10,
    createRenderer: options => new RenderedMapD()
};
const extensions = [
    {
        id: 'jupyterlab-mapd:factory',
        rendererFactory: exports.rendererFactory,
        dataType: 'string'
    }
];
exports.default = extensions;
/**
 * A namespace for private data.
 */
var Private;
(function (Private) {
    /**
     * Create an image node from a base64-encoded image.
     */
    function createImageNode(blob) {
        const img = document.createElement('img');
        let blobUrl = `data:${exports.IMAGE_MIME};base64,${blob}`;
        img.src = blobUrl;
        return img;
    }
    Private.createImageNode = createImageNode;
})(Private || (Private = {}));
